@@ -12,8 +12,9 @@ from ._utils.utils import *
 import seaborn as sns
 import matplotlib.pyplot as plt
 from datetime import datetime, timedelta
-from searchtweets import collect_results, load_credentials
+#from searchtweets import collect_results, load_credentials
 from .geomethods import TweetGeoGenerator
+from .apicall import retrieve_tweets
 
 from wordcloud import WordCloud, STOPWORDS
 
@@ -86,9 +87,7 @@ class TweetDownloader:
         self.authors_df = None
         self.places_df = None
         self.replies_df = None
-        self.search_args = load_credentials(self.credentials,
-                                            yaml_key="search_tweets_v2",
-                                            env_overwrite=False)
+        #self.search_args = load_credentials(keys_path=self.credentials)
         self.timestamp = None
 
     def tweets_from_query(self, query_params, max_page, save_temp, max_tweets, reply_mode=False):
@@ -113,24 +112,27 @@ class TweetDownloader:
             # Collect results according to query parameters, tweets per page...
             # ... and authentication credentials
 
-            tweets_page = collect_results(query_params, max_tweets=max_page,
-                                          result_stream_args=self.search_args)
+            #tweets_page = collect_results(query_params, max_tweets=max_page,
+            #                              result_stream_args=self.search_args)
+
+            tweets_page = retrieve_tweets(query_params=query_params, keys_path=self.credentials)
+
             if len(tweets_page) != 0:  # ensures we don't process a blank page
 
                 # Adds retrieved page of tweets to list of pages
                 list_tweet_pages.append(tweets_page)
 
                 # Adds number of retrieved tweets to tweet count
-                tweet_count += tweets_page[0]['meta']['result_count']
+                tweet_count += tweets_page['meta']['result_count']
 
-                df_page = pd.DataFrame(tweets_page[0]['data'])
-                df_page_authors = pd.DataFrame(tweets_page[0]['includes']['users'])
+                df_page = pd.DataFrame(tweets_page['data'])
+                df_page_authors = pd.DataFrame(tweets_page['includes']['users'])
 
                 df_tweets = pd.concat([df_tweets, df_page])
                 df_authors = pd.concat([df_authors, df_page_authors])
 
                 try:
-                    df_page_places = pd.DataFrame(tweets_page[0]['includes']['places'])
+                    df_page_places = pd.DataFrame(tweets_page['includes']['places'])
                     df_places = pd.concat([df_places, df_page_places])
                 except KeyError:
                     # print('No places on this page...')
@@ -165,9 +167,9 @@ class TweetDownloader:
                 try:
                     if (not reply_mode):
                         print('Ending page %s with next_token=%s. %s tweets retrieved (%s total)' % (
-                            page_count, tweets_page[0]['meta']['next_token'], tweets_page[0]['meta']['result_count'],
+                            page_count, tweets_page['meta']['next_token'], tweets_page['meta']['result_count'],
                             tweet_count))
-                    next_token = tweets_page[0]['meta']['next_token']
+                    next_token = tweets_page['meta']['next_token']
                     query_params['next_token'] = next_token
                     page_count += 1
                     time.sleep(5)
@@ -194,7 +196,38 @@ class TweetDownloader:
                    end_time=datetime.now().strftime("%Y-%m-%dT%H:%M:%SZ"),
                    lang=None, include_retweets=False, place=None,
                    max_tweets=10, max_page=500, save_temp=True, save_final=True,
-                   save_replies=False, include_replies=False, max_replies=10, temp_replies=True, ):
+                   save_replies=False, include_replies=False, max_replies=10, temp_replies=True):
+        """
+        Parameters
+        ----------
+        query : str
+            Words to be searched in tweets. Twitter API query operators supported.
+        start_time : str
+            Lower bound of  time frame in which tweets are going to be searched in date-time format (default is current date and time minus 24 hours)
+        end_time : str
+            Upper bound of time frame in which tweets are going to be searched in date-time format (default is current date and time time)
+        lang : str, optional
+            Two letter code for language to be imposed in retrieved tweets
+        include_retweets : bool
+            Whether to include tweets that are just a retweet of a previous one (default is False)
+        place : str, optional
+            Two letter code for country or place in which the search is going to be constraint
+        max_tweets : int
+            The maximum amount of tweets to retrieve in total (default is 10)
+        max_page : int
+            The maximum amount of tweets allowed per tweets page (default is 500)
+        save_temp : bool
+            Whether to save current progress (default is True)
+        save_final : bool
+            Whether to save final tweets dataframe after download is over (default is True)
+        save_replies : bool
+            Whether to include the replies to the downoaded tweets (default is false)
+        max_replies : bool
+            Maximum amount of replies per tweet if replies are allowed (default is 10)
+        temp_replies : bool
+            Whether to save progress while downloading replies if these are allowed (default is True)
+        """
+
 
         # Query parameters
         has_geo = True  # This could be introduced as func parameter in a future version
@@ -230,6 +263,8 @@ class TweetDownloader:
 
         filename = os.path.join(self.output_folder, self.name)
 
+        print(f'Tweets download done. A total of {self.tweets_df.shape[0]} tweets were retrieved.')
+
         if save_final:
             self.tweets_df['place_id'] = self.tweets_df.geo.apply(lambda x: get_attribute_from_dict(x, 'place_id'))
             # Creates date column in date format
@@ -267,18 +302,28 @@ class TweetDownloader:
         #    return self.tweets_df, self.places_df, self.authors_df,
 
     def get_replies(self, max_replies=10, save_temp=True, save_final=True):
+        """
+        Parameters
+        ----------
+        max_replies : int
+            Maximum number of replies for each tweet in the original tweets dataset (default is 10)
+        save_temp : bool
+            Whether to save progress at each page (default is True)
+        save_final : bool,
+            Whether to save final replies dataset (default is True)
+        """
 
         df_tweets_rep = pd.DataFrame()
 
         print('Downloading replies... this might take some time')
 
-        total_tweets = self.tweets_df.shape[0]
+        total_tweets = self.tweets_df.query('replies>0').shape[0]
         rep_count = 0
         total_replies = 0
 
         filename = os.path.join(self.output_folder, self.name)
 
-        for conversation_id in self.tweets_df.conversation_id:
+        for conversation_id in self.tweets_df.query('replies>0').conversation_id:
             rep_count += 1
 
             print(f'getting replies for tweet {rep_count} out of {total_tweets} (total replies so far:{total_replies})')
@@ -308,11 +353,17 @@ class TweetDownloader:
 
             total_replies = df_tweets_rep.shape[0]
 
+            if total_replies >= max_replies:
+                break
+                print('The desired maximum amount of replies was reached.')
+
         if save_final:
             df_tweets_rep.to_csv(f'{filename}_replies_{self.timestamp}', index=False)
 
         self.replies_df = df_tweets_rep
 
+        print(f'Replies download done. {total_replies} reply tweets were downloaded')
+
         return df_tweets_rep
 
     def build_dataframes(self):
@@ -348,6 +399,17 @@ class TweetDownloader:
         print('Done')
 
     def tweets_from_csv(self, path, sep=',', save_temp=True):
+        """
+        Parameters
+        ----------
+        path : str
+            The path to the csv path containing the download parameters
+        sep : str, optional
+            The separator of the csv file (default is ,)
+        save_temp : bool, optional
+            Whether to save or not progress at each downloaded page (default is True)
+        """
+
         # loading config values into a DataFrame
         df_param = pd.read_csv(path, sep=sep)
 
@@ -361,9 +423,25 @@ class TweetDownloader:
         plot_wordcloud = df_param.query('parameter=="wordcloud"')['value'].item()
         plot_barplot = df_param.query('parameter=="barplot"')['value'].item()
         custom_stopwords = df_param.query('parameter=="stopwords"')['value'].item()
+        lang = df_param.query('parameter=="language"')['value'].item()
+        place = df_param.query('parameter=="place"')['value'].item()
+        include_retweets = df_param.query('parameter=="include_retweets"')['value'].item()
+        has_geo = df_param.query('parameter=="only_georreferenced"')['value'].item()
 
         self.name = filename
 
+        if pd.isna(place): place = None
+        if pd.isna(lang): lang = None
+
+        if not pd.isna(lang):
+            query += f' (lang:{lang})'
+        if not pd.isna(place):
+            query += f' (place_country:{place})'
+        if has_geo == 'yes':
+            query += ' (has:geo)'
+        if include_retweets != 'yes':
+            query += ' (-is:retweet)'
+
         # Query parameters. See the parameters.csv to modify or see the description
         # of each parameter
         query_params = {'query': query,
@@ -396,11 +474,27 @@ class TweetDownloader:
                            bar_plot=plot_barplot, save_bar_plot=True)
 
     def tweets_to_shp(self, save_path='', geo_type='centroids'):
+        """
+        Parameters
+        ----------
+        path : str
+            The path to save the exported shapefile (default is current path)
+        geo_type : {'centroids', 'bbox'}, optional
+            The type of geometry (default is centroids)
+        """
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.save_tweets_shp(save_path, geo_type)
 
     def places_to_shp(self, save_path='', geo_type='centroids'):
+        """
+        Parameters
+        ----------
+        path : str
+            The path to save the exported shapefile (default is current path)
+        geo_type : {'centroids', 'bbox'}, optional
+            The type of geometry (default is centroids)
+        """
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.save_places_shp(save_path, geo_type)
@@ -411,31 +505,70 @@ class TweetDownloader:
         tgeo.simple_tweets_map()
 
     def interactive_map(self):
+        print('Generating map...')
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.plot_tweets_points()
+        print('Done. Map will be displayed in browser')
 
     def interactive_map_agg(self):
+        print('Generating map...')
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.plot_tweets_aggregated()
+        print('Done. Map will be displayed in browser')
 
     def plot_heatmap(self, radius=20):
+        """
+        Parameters
+        ----------
+        radius : int
+            The radius of the heatmap plot (default is 20)
+        """
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.plot_tweets_heatmap(radius)
 
     def map_animation(self, time_unit):
+        """
+        Parameters
+        ----------
+        time_unit : {'second', 'minute', 'hour', 'day', 'month', 'year'}
+            Time unit to aggregate by (default is 'day')
+        """
+        print('Generating map animation...')
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
         tgeo.bubble_animation(time_unit)
+        print('Done. Animation will be displayed in browser')
 
     def wordcloud(self, custom_stopwords=None, background_color='black', min_word_length=4,
-                  save_wordcloud=False, save_path='', bar_plot=False, save_bar_plot=False):
+                  save_wordcloud=True, bar_plot=False, save_bar_plot=False):
+
+        """
+        Parameters
+        ----------
+        custom_stopwords : list
+            List of words to exclude from word cloud
+        background_color : {'black', 'white'}
+            Background color of wordcloud plot
+        min_word_length : int
+            Minimum length of strings to be considered for word cloud (default is 4)
+        save_wordcloud: bool,
+            Whether to save plot (default is True)
+        bar_plot: bool
+            Whether to display barplot with word frequency (default is False)
+        save_bar_plot:
+            Whether to save barplot (default is False)
+        """
+
         if custom_stopwords is None:
             custom_stopwords = []
         plt.rcParams.update({'font.size': 12})
 
+        #custom_stopwords.append('http')
+        #custom_stopwords.append('https')
+
         df_tweets = self.tweets_df.copy()
 
         # gets all text contained in tweets
@@ -460,8 +593,8 @@ class TweetDownloader:
         ax.axes.xaxis.set_visible(False)
         ax.axes.yaxis.set_visible(False)
         if save_wordcloud:
-            plt.savefig(os.path.join(save_path, f'{self.name}_wordcloud.png'))
-            print('Wordcloud saved at', os.path.join(save_path, f'{self.name}_wordcloud.png'))
+            plt.savefig(os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
+            print('Wordcloud saved at', os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
         else:
             plt.show()
 
@@ -478,11 +611,13 @@ class TweetDownloader:
             plt.yticks(fontsize=40)
             plt.xlabel("freq", fontsize=50)
             plt.ylabel("word", fontsize=50)
+            plt.savefig(os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
+
             if save_bar_plot:
                 sns.barplot(df_wordcount.freq[:20],
-                            df_wordcount.word[:20]).get_figure().savefig(os.path.join(save_path,
+                            df_wordcount.word[:20]).get_figure().savefig(os.path.join(self.output_folder,
                                                                                       f'{self.name}_barplot.png'))
-                print('Barplot saved at', os.path.join(save_path, f'{self.name}_barplot.png'))
+                print('Barplot saved at', os.path.join(self.output_folder, f'{self.name}_barplot.png'))
             else:
                 sns.barplot(df_wordcount.freq[:20],
                             df_wordcount.word[:20]).get_figure()
